/*
 * FrameControlField.cpp
 *
 *  Created on: Jan 25, 2016
 *      Author: badenhopcw
 */

#include "FrameControl.h"
#include <cstring>
#include <arpa/inet.h>

namespace Zwave {

bool
FrameControl::IsSinglecastFrame ()
{
	return (this->GetFrameType () == FrameControl::FRAME_TYPE_SINGLECAST);
}

bool
FrameControl::IsMulticastFrame ()
{
	return (this->GetFrameType () == FRAME_TYPE_MULTICAST);
}

bool
FrameControl::IsAcknowledgement ()
{
	return (this->GetFrameType () == FRAME_TYPE_ACK);
}

bool
FrameControl::IsRouted ()
{
	return ((m_data & FrameControl::ROUTED_BIT) > 0);
}

bool
FrameControl::AckRequested ()
{
	return ((m_data & FrameControl::ACKREQ_BIT) > 0);
}

bool
FrameControl::IsLowPower ()
{
	return ((m_data & FrameControl::LOWPOW_BIT) > 0);
}

bool
FrameControl::IsSpeedModified ()
{
	return ((m_data & FrameControl::SPDMOD_BIT) > 0);
}


bool
FrameControl::SenderWakesOnRepeatedBeam ()
{
	return (this->GetBeamControl () != BEAM_TYPE_AWAKE);
}

bool
FrameControl::SenderWakesOnShortBeam ()
{
	return (this->GetBeamControl () == BEAM_TYPE_WAKEONSHORT);
}

bool
FrameControl::SenderWakesOnLongBeam ()
{
	return (this->GetBeamControl () == BEAM_TYPE_WAKEONLONG);
}

uint8_t
FrameControl::GetSequenceNumber ()
{
	return (uint8_t)(m_data & FrameControl::SEQNBR_MASK >> FrameControl::SEQNBR_SHIFT);
}

FrameControl::FrameType
FrameControl::GetFrameType ()
{
	return (FrameType)((m_data & FrameControl::FRMTYP_MASK) >> FrameControl::FRMTYP_SHIFT);
}

FrameControl::BeamType
FrameControl::GetBeamControl ()
{
	return (BeamType)((m_data & FrameControl::BEMCTL_MASK) >> FrameControl::BEMCTL_SHIFT);
}

void
FrameControl::SetAckRequestedFlag ()
{
	m_data |= FrameControl::ACKREQ_BIT;
}

void
FrameControl::UnsetAckRequestedFlag ()
{
	m_data &= ~FrameControl::ACKREQ_BIT;
}

void
FrameControl::SetRoutedFlag ()
{
	m_data |= FrameControl::ROUTED_BIT;
}

void
FrameControl::UnsetRoutedFlag ()
{
	m_data &= ~FrameControl::ROUTED_BIT;
}

void
FrameControl::SetLowPowerFlag ()
{
	m_data |= FrameControl::LOWPOW_BIT;
}

void
FrameControl::UnsetLowPowerFlag ()
{
	m_data &= ~FrameControl::LOWPOW_BIT;
}

void
FrameControl::SetSpeedModifiedFlag ()
{
	m_data  |= FrameControl::SPDMOD_BIT;
}

void
FrameControl::UnsetSpeedModifiedFlag ()
{
	m_data &= ~FrameControl::SPDMOD_BIT;
}

void
FrameControl::SetWakeOnShortBeamFlag ()
{
	m_data |= FrameControl::WKONSH_BIT;
}

void
FrameControl::UnsetWakeOnShortBeamFlag ()
{
	m_data &= ~FrameControl::WKONSH_BIT;
}

void
FrameControl::SetWakeOnLongBeamFlag ()
{
	m_data |= FrameControl::WKONLG_BIT;
}

void
FrameControl::UnsetWakeOnLongBeamFlag ()
{
	m_data &= ~FrameControl::WKONLG_BIT;
}

FrameControl::FrameControl() {
	// TODO Auto-generated constructor stub

}

FrameControl::FrameControl(uint16_t fc) {
	m_data = ::ntohs(fc);
}
FrameControl::~FrameControl() {
	// TODO Auto-generated destructor stub
}

uint16_t
FrameControl::ExportPackedField ()
{
	return ::htons(m_data);
}

void
FrameControl::SetSequenceNumber (uint8_t seqnbr)
{
	//m_data |= (uint16_t)((seqnbr & FrameControl::SEQNBR_MASK) << FrameControl::SEQNBR_SHIFT);
	uint16_t val = ((uint16_t)seqnbr << FrameControl::SEQNBR_SHIFT); // Load and shift to position

	m_data &= ~FrameControl::SEQNBR_MASK; // Clear bits
	m_data |= (val & FrameControl::SEQNBR_MASK);
	
}

void
FrameControl::SetFrameType (FrameType type)
{
	//m_data |= (uint16_t)(((uint16_t)type & FrameControl::FRMTYP_MASK) << FrameControl::FRMTYP_SHIFT);
	uint16_t val = ((uint16_t)type << FrameControl::FRMTYP_SHIFT);
	m_data &= ~FrameControl::FRMTYP_MASK;
	m_data |= (val & FrameControl::FRMTYP_MASK);
}


} /* namespace Zwave */
