/*
 * FrameControlField.h
 *
 *  Created on: Jan 25, 2016
 *      Author: badenhopcw
 */

#ifndef FRAMECONTROL_H_
#define FRAMECONTROL_H_

#include <stdint.h>

namespace Zwave {



class FrameControl {
public:
	static const uint8_t FRAME_TYPE_SINGLECAST = 0x01;
	static const uint8_t FRAME_TYPE_MULTICAST = 0x2;
	static const uint8_t FRAME_TYPE_ACK = 0x03;

	static const uint8_t BEAM_TYPE_AWAKE = 0x00;
	static const uint8_t BEAM_TYPE_WAKEONSHORT = 0x01;
	static const uint8_t BEAM_TYPE_WAKEONLONG = 0x02;
	static const uint8_t SEQNBR_MOD = 16;

	typedef uint8_t FrameType;
	typedef uint8_t BeamType;

	//enum class FrameType {SINGLECAST = 0x01, MULTICAST = 0x02, ACK = 0x03};
	//enum class BeamType {AWAKE = 0x00, WAKEONSHORTBEAM = 0x01, WAKEONLONGBEAM = 0x02, UNKNOWN = 0x03};
	FrameControl ();
	FrameControl (uint16_t fc);
	virtual ~FrameControl ();

	//FrameControlField * operator= (const FrameControlField &a);

	bool IsSinglecastFrame ();
	bool IsMulticastFrame ();
	bool IsAcknowledgement ();
	//bool IsCfg3RoutedFrame (); // Channel conf 3 (if used) should extend this class
	bool AckRequested ();
	bool IsRouted ();
	bool IsLowPower ();
	bool IsSpeedModified ();
	bool SenderWakesOnRepeatedBeam (); // No Set/Unset methods for this because it spec indicates it exists with either long or short beam mode, never alone
	bool SenderWakesOnShortBeam ();  // implicit is to also wake on Repeated beam
	bool SenderWakesOnLongBeam (); // implicit is to also wake on repeated beam
	uint8_t GetSequenceNumber ();
	FrameType GetFrameType ();

	void SetSequenceNumber (uint8_t seqnbr);
	void SetFrameType (FrameType type);

	void SetAckRequestedFlag ();
	void UnsetAckRequestedFlag ();
	void SetRoutedFlag ();
	void UnsetRoutedFlag ();
	void SetLowPowerFlag ();
	void UnsetLowPowerFlag ();
	void SetSpeedModifiedFlag ();
	void UnsetSpeedModifiedFlag ();
	void SetWakeOnShortBeamFlag ();
	void UnsetWakeOnShortBeamFlag ();
	void SetWakeOnLongBeamFlag ();
	void UnsetWakeOnLongBeamFlag ();
	uint16_t ExportPackedField ();
	//virtual uint8_t * Serialize ();
	//virtual size_t Deserialize (uint8_t *buf, size_t len);
	//virtual size_t GetSerializedLength ();

	//FrameControlField * operator= (const FrameControlField &a);

	static const uint16_t ROUTED_BIT=0x8000;
	static const uint16_t ACKREQ_BIT=0x4000;
	static const uint16_t LOWPOW_BIT=0x2000;
	static const uint16_t SPDMOD_BIT=0x1000;
	static const uint16_t WKONLG_BIT=0x0040;
	static const uint16_t WKONSH_BIT=0x0020;

protected:
	BeamType GetBeamControl ();
	static const uint16_t BEMCTL_MASK=0x0060;
	static const uint16_t BEMCTL_SHIFT=5;
	static const uint16_t SEQNBR_MASK=0x000F;
	static const uint16_t SEQNBR_SHIFT=0;
	static const uint16_t FRMTYP_MASK=0x0F00;
	static const uint16_t FRMTYP_SHIFT=8;

	//virtual void SerializeChain (std::vector<uint8_t> *chain);

	uint16_t m_data;
};

} /* namespace Zwave */
#endif /* FRAMECONTROLFIELD_H_ */
