/*
 * MAC.cpp
 *
 *  Created on: Jan 29, 2016
 *      Author: badenhopcw
 */

#include "MAC.h"
#include "FrameControl.h"
#include <cstring>
#include <iostream>

namespace Zwave {

MAC::MAC() {
	// TODO Auto-generated constructor stub

}

MAC::~MAC() {
	// TODO Auto-generated destructor stub
}


uint8_t
MAC::GenerateChecksum (uint8_t *mpdu, size_t len)
	{
			size_t i=0;
			uint8_t sum=0xFF;

			if (len == 0)
				return 0x00;

			if (len > MAC::MAX_MPDU_SIZE)
				len = MAC::MAX_MPDU_SIZE;

			for (i=0;i<len-1;i++)
				sum = sum ^ mpdu[i];  // XOR (from ITU G9959)

			return sum;
	}


bool
MAC::CheckChecksum (uint8_t *mpdu, size_t len)
	{			
			size_t i=0;
			uint8_t sum = 0xFF;

			if (len > MAC::MAX_MPDU_SIZE)
				len = MAC::MAX_MPDU_SIZE;

			for (i=0;i<len;i++)
				sum = sum ^ mpdu[i];  // XOR (from ITU G9959)

			if (sum != 0x00){
				std::cout << "Len=" << len << " Calculated Sum = " << (uint16_t)sum << std::endl;
				return false;
			}else {
				return true;
			}
	}

void
MAC::CreateSinglecastMHR (mhr_single_t &mhr, uint32_t homeid, uint8_t src, uint8_t dst, uint16_t frame_ctrl, uint8_t seqnbr)
	{
		FrameControl fc;

		mhr.homeId = homeid;
		mhr.srcId = src;
		mhr.dstId = dst;
		fc = FrameControl (frame_ctrl);
		fc.SetFrameType(FrameControl::FRAME_TYPE_SINGLECAST);
		fc.SetSequenceNumber(seqnbr);
		mhr.frameCtl = fc.ExportPackedField ();

		return;
	}

void
MAC::CreateSinglecastReplyMHR (mhr_single_t &mhr, mhr_single_t *srcMHR, uint8_t seqnbr)
	{
		FrameControl fc;
		uint8_t src = srcMHR->srcId;
		mhr = *srcMHR;

		mhr.srcId = mhr.dstId;
		mhr.dstId = src;

		fc = FrameControl (mhr.frameCtl);
		fc.SetFrameType (FrameControl::FRAME_TYPE_SINGLECAST);
		fc.SetSequenceNumber (seqnbr);
		mhr.frameCtl = fc.ExportPackedField ();

		return;
	}

void
MAC::CreateAckReplyMHR (mhr_single_t &mhr, mhr_single_t *srcMHR)
	{
		FrameControl fc = FrameControl (srcMHR->frameCtl);
		fc.SetFrameType (FrameControl::FRAME_TYPE_ACK);
		fc.UnsetAckRequestedFlag (); //To be sure
		
		mhr.homeId = srcMHR->homeId;
		mhr.srcId = srcMHR->dstId; // flip src and dst
		mhr.frameCtl = fc.ExportPackedField ();
		mhr.length = MAC::MSDU_ACK_SIZE;
		mhr.dstId = srcMHR->srcId;

		return;
	}

void
MAC::CreateAckReplyMPDU (mpdu_ack_t &mpdu, mhr_single_t *srcMHR)
	{

		CreateAckReplyMHR (mpdu.mhr, srcMHR);
		mpdu.checksum = GenerateChecksum ((uint8_t*)&mpdu, mpdu.mhr.length);

		return;
	}

size_t
MAC::GetFixedMHR (mhr_fixed_t &mhr, uint8_t *frame, size_t len)
	{
		size_t size = sizeof(mhr_fixed_t);
		if (len < size)
			return 0;

		std::memcpy (&mhr, frame, size);

		return len;
	}

size_t
MAC::GetSinglecastMHR (mhr_single_t &mhr, uint8_t *frame, size_t len)
	{
		FrameControl fc;
		mhr_fixed_t fHdr;

		size_t size = sizeof(mhr_single_t);

		if (len < size)
			return 0;

		GetFixedMHR(fHdr, frame, size);
		fc = FrameControl (fHdr.frameCtl);
		
		
		if (fc.IsSinglecastFrame () || fc.IsAcknowledgement ())	{
			memcpy (&mhr, frame, size);
			return size;
		}else{
			memset (&mhr, 0, sizeof(mhr_single_t));
			return 0;
		}

	}

void 
MAC::GetFrameControl (FrameControl &fc, uint8_t* mpdu, size_t len)
	{
		mhr_fixed_t mhr;

		MAC::GetFixedMHR (mhr, mpdu, len);
		fc = FrameControl (mhr.frameCtl);
	}

} /* namespace Zwave */
