/*
 * MAC.h
 *
 *  Created on: Jan 29, 2016
 *      Author: badenhopcw
 */

#ifndef MAC_H_
#define MAC_H_

#include <stdint.h>
#include <string>
#include "FrameControl.h"

// Ignoring retries and cdma/ca stuff for now

namespace Zwave{


class MAC {
public:
	static const size_t MAX_MSDU_SINGLE_PAYLOAD_SIZE = 54;
	static const size_t MAX_MSDU_MULTI_PAYLOAD_SIZE = 29;
	static const size_t MSDU_ACK_SIZE = 10;
	static const size_t MAX_MPDU_SIZE = 64;
	static const size_t MIN_MPDU_SIZE = MSDU_ACK_SIZE;
	static const size_t MAX_BITFIELD_SIZE = 29;
	static const size_t CHECKSUM_SIZE = 1;
	static const size_t BROADCAST_NODEID = 0xFF;

	struct mhr_fixed_t{
		uint32_t homeId;
		uint8_t srcId;
		uint16_t frameCtl;
		uint8_t length;
	} __attribute__((packed));

	struct mhr_single_t {
		uint32_t homeId;
		uint8_t srcId;
		uint16_t frameCtl;
		uint8_t length;
		uint8_t dstId;
	} __attribute__((packed));

	struct mhr_multi_t{
		uint32_t homeId;
		uint8_t srcId;
		uint16_t frameCtl;
		uint8_t length;		
		uint8_t mcast_ctrl;
		uint8_t mcast_bitmsk[MAC::MAX_BITFIELD_SIZE];
	} __attribute__((packed));

	struct mpdu_ack_t{
		struct mhr_single_t mhr;
		uint8_t checksum;
	} __attribute__((packed));

	struct mpdu_single_t{
		struct mhr_single_t mhr;
		uint8_t payload[MAC::MAX_MSDU_SINGLE_PAYLOAD_SIZE + 1]; // +1 for checksum
	}__attribute__((packed));

	struct mpdu_multi_t{
		struct mhr_multi_t mhr;
		uint8_t payload[MAC::MAX_MSDU_MULTI_PAYLOAD_SIZE + 1]; // + 1 for checksum
	}__attribute__((packed));



	MAC();
	virtual ~MAC();

	static uint8_t GenerateChecksum (uint8_t* mpdu, size_t len);
	static bool CheckChecksum (uint8_t *mpdu, size_t len);
	static void CreateSinglecastMHR (mhr_single_t &mhr, uint32_t homeid, uint8_t src, uint8_t dst, uint16_t frame_ctrl, uint8_t seqnbr);
	static void CreateSinglecastReplyMHR (mhr_single_t &mhr, mhr_single_t *srcMHR, uint8_t seqnbr);
	static void CreateAckReplyMHR (mhr_single_t &mhr, mhr_single_t *srcMHR);
	static void CreateAckReplyMPDU (mpdu_ack_t &mpdu, mhr_single_t *srcMHR);
	static size_t GetFixedMHR(mhr_fixed_t &mhr, uint8_t *frame, size_t len);
	static size_t GetSinglecastMHR (mhr_single_t &mhr, uint8_t *frame, size_t len);
	static void GetFrameControl (FrameControl &fc, uint8_t* mpdu, size_t len);
};

} /* namespace Zwave */
#endif /* MAC_H_ */
