/*
 * MacIface.h
 *
 *  Created on: Feb 1, 2016
 *      Author: badenhopcw
 */

#ifndef MACIFACE_H_
#define MACIFACE_H_

#include "PhyIface.h"
#include "MAC.h"
#include "Timer.h"
#include <stdint.h>

namespace Zwave {

class MacIface {
public:
	static const ssize_t RECVFRAME_TIMEOUT = -2;
	static const ssize_t RECVFRAME_BUFOFLOW = -3;
	static const ssize_t SENDFRAME_ACK = -4;
	static const ssize_t SENDFRAME_NOACK = -5;

	static const size_t LONGCONT_NUM_BEAMS = 250;
	static const size_t SHORTCONT_NUM_BEAMS = 75;
	static const uint8_t FILT_NODEID_ANY = MAC::BROADCAST_NODEID;
	static const uint32_t FILT_HOMEID_ANY = 0;
	static const uint8_t FILT_SEQNBR_ANY = 0x11;
	static const uint8_t FILT_FRAMETYPE_ANY = 0xFF;

	static const size_t MAX_SEND_ATTEMPTS = 3;
	static const uint32_t BACKOFF_WAIT[MAX_SEND_ATTEMPTS];
	static const uint32_t BEAM_WAIT[MAX_SEND_ATTEMPTS];

	MacIface();
	MacIface (PhyIface *phy, uint32_t homeid, uint8_t nodeid, bool reqValidChecksum);
	MacIface(PhyIface *phy);
	virtual ~MacIface();

	uint32_t GetHomeId_HostEnd ();
	uint32_t GetHomeId_NetEnd ();
	uint8_t GetNodeId ();

	void SetHomeId_HostEnd (uint32_t homeid);
	void SetHomeId_NetEnd (uint32_t homeid);
	void SetNodeId (uint8_t nodeid);

	// Recv is the big daddy that checks for a variety of conditions for a certain duration. If it finds something that matches your requirements it returns. Or if it times out it returns (check return value). It will also indicate if the caller didn't provide enough room to copy the entire frame as a different error condition return value.
	ssize_t Recv (uint8_t *buf, size_t maxlen, uint32_t homeid, uint8_t src, uint8_t dst, uint8_t frameType, uint8_t seqnbr, Timer *timer);
	ssize_t RecvAny(uint8_t *buf, size_t maxlen, Timer *timer); // Get any Z-Wave frame
	ssize_t RecvNetworkAny (uint8_t *buf, size_t maxlen, Timer *timer); // Get any Z-Wave from from our network
	ssize_t RecvNetwork (uint8_t *buf, size_t maxlen, Timer *timer); // Get any Z-Wave from from our network to my nodeId
	ssize_t RecvNetworkFrom (uint8_t *buf, size_t maxlen, uint8_t from, Timer *timer); // Get any Z-Wave from from our network to me from dst nodeId
	ssize_t RecvAckFrom (uint8_t from, uint8_t seqnbr, Timer *timer); // Use this to wait a certain amount of time for an ack reply. Its kind of wastful with the buffers, but the interface is consistent with Recv()

	void SendFrame (uint8_t *frame, size_t len);
	ssize_t SendFrameWaitForAck (uint8_t *frame, size_t len);
	ssize_t SendBeamedFrameWaitForAck (uint8_t *frame, size_t len, uint8_t wakupTarget);

protected:
	static const size_t BUFFER_SIZE = MAC::MAX_MPDU_SIZE;
	PhyIface *m_phy;
	uint32_t m_homeId;
	uint8_t m_nodeId;
	bool m_reqValidChecksum;
	uint8_t m_buf[BUFFER_SIZE];
};

} /* namespace Zwave */
#endif /* MACIFACE_H_ */
