/*
 * ScapyRadioIface.cpp
 *
 *  Created on: Jan 29, 2016
 *      Author: badenhopcw
 */

#include "PhyIface.h"
#include "ScapyRadioIface.h"
#include "MAC.h"
#include "Timer.h"
#include <cstring>
#include <iostream>
#include <stdio.h>
#include <sys/select.h>
#include <stdint.h>

namespace Zwave {

//const uint8_t ScapyRadioIface::HDR[HEADER_LENGTH] = { MAGIC_NUMBER, 0,0,0,0,0,0,0};
const uint8_t ScapyRadioIface::HDR[HEADER_LENGTH] = { MAGIC_NUMBER, 0,0,0,0,0,0,0}; // For a test

ScapyRadioIface::ScapyRadioIface (std::string TXipAddress, uint16_t TXudpPort, 
																	std::string RXipAddress, uint16_t RXudpPort)
{
	 m_udpClient = UdpSocketClientIface(TXipAddress, TXudpPort, 0);
	 m_udpServer = UdpSocketServerIface (RXipAddress, RXudpPort, 0);

	m_nfds = m_udpServer.GetSocket () + 1; // Max FD for select(). Set this here
}

ScapyRadioIface::ScapyRadioIface() {

}

ScapyRadioIface::~ScapyRadioIface() {
}


void
ScapyRadioIface::SendFrame (const uint8_t *frame, const size_t len, size_t numBeams, uint8_t beamTarget)
{

	memcpy (m_buf, ScapyRadioIface::HDR, ScapyRadioIface::HEADER_LENGTH);
	memcpy (&m_buf[ScapyRadioIface::HEADER_LENGTH], frame, len);

	if (numBeams > 0)
		{
			m_buf[1] = numBeams;
			m_buf[2] = beamTarget;
		}

	if (m_udpClient.SendTo (m_buf, len + ScapyRadioIface::HEADER_LENGTH) < 0) {
		perror("Scapy Radio SendTo Failed");
		exit (1);
	} 

	return;
}

ssize_t
ScapyRadioIface::RecvFrame (uint8_t *buf, const size_t maxlen, Timer *timer)
{
	// Loop until Recv any valid or timeout deadline reached
	
	ssize_t bytesRead = 0;
	int result;
	struct timeval tv;
	size_t attempts = 0;

	while (!timer->HasTimeExpired ())
		{	
			timer->GetRemainingTime(tv);
			//std::cout << "PHY read attempt: " << attempts++ << std::endl;
			//std::cout << "PHY::Time left: " << tv.tv_sec << "." << tv.tv_usec << std::endl;

			// CWB: Learned that this gets modified by the select call so we must reset it before every select call
			FD_ZERO(&m_selectFDs);
			FD_SET(m_udpServer.GetSocket (),&m_selectFDs);
			
			result = select (m_nfds, &m_selectFDs, NULL, NULL, &tv);
			//std::cout << "Select returned with value " << result << " on socket " << m_udpServer.GetSocket () << std::endl;
			if (result == -1)
				{
					perror ("select ()");
					break;
				}
			else if (result)
				{
					//std::cout << "FD_ISSET = " << FD_ISSET (m_udpServer.GetSocket (), &m_selectFDs) << std::endl;
					if (FD_ISSET (m_udpServer.GetSocket (), &m_selectFDs)) // frame waiting for us
						{  //CWB: Do we need to force this to be nonblocking just in case? Or does select never lie?
							bytesRead = m_udpServer.RecvFrom (m_buf, ScapyRadioIface::BUFFER_SIZE);
							//std::cout << " Bytes read: " << bytesRead << std::endl;
							if (bytesRead > 0)
								{
									// I need to validate the header and extract the payload

									// Check for magic number
									if (m_buf[0] != ScapyRadioIface::MAGIC_NUMBER)
										{
											//std::cout << "Frame does not have magic number header\n";
											continue;
										}
									else
										{
											// We found a good frame. Ignore if it is a beam
											if ( m_buf[ScapyRadioIface::HEADER_LENGTH] != 0x55)
												{
													std::memcpy (buf, &m_buf[ScapyRadioIface::HEADER_LENGTH], bytesRead - ScapyRadioIface::HEADER_LENGTH);
													return ((size_t)bytesRead - ScapyRadioIface::HEADER_LENGTH);
												}
										}
								}
				}
			// else timeout has occured. Loop again to check if there is still time...probably not but it can't hurt to check
		}
	
	//std::cout << "PHY:Timeout\n";

	return PhyIface::RECVFRAME_TIMEOUT;
}

/*
	do {		
		bytesRead = m_udpServer.RecvFrom (m_buf, ScapyRadioIface::BUFFER_SIZE);
		//std::cout << "Read " << bytesRead << " bytes.\n";

		if (bytesRead > 0)
		{

			// I need to validate the header and extract the payload

			// Check for magic number
			if (m_buf[0] != ScapyRadioIface::MAGIC_NUMBER)
			{
				std::cout << "Frame does not have magic number header\n";
				bytesRead = 0;
				continue;
			}
			
// CWB: This should be done in MAC iface
			if (bytesRead < (MAC::MSDU_ACK_SIZE + ScapyRadioIface::HEADER_LENGTH)) // If smaller than ACK packet, it isn't a zwave frame
			{
				std::cout << "Frame is fragmented\n";
				bytesRead = 0;
				continue;
			}
			

		} else {
			perror("RecvFrom UDP socket error");
			exit (2);
		}

	} while (bytesRead <= 0);

	// Found something, so hack off the header and return
	memcpy (buf, &m_buf[ScapyRadioIface::HEADER_LENGTH], bytesRead - ScapyRadioIface::HEADER_LENGTH);
	return (size_t)bytesRead - ScapyRadioIface::HEADER_LENGTH;
 */
}

} /* namespace Zwave */
