#ifndef _APP_SECURITY_H_
#define _APP_SECURITY_H_

#include <stdint.h>
//include <stddef.h>
#include "MAC.h"
#include "APP.h"

namespace Zwave {

class EncPayload {
public:
		static const size_t IV_SIZE = 8;
		static const size_t CMAC_SIZE = 8;
		static const size_t MIN_ENCAP_PAYLOAD_SIZE = 3;
		static const size_t MAX_SIZE = MAC::MAX_MSDU_SINGLE_PAYLOAD_SIZE;
		static const size_t MIN_SIZE = EncPayload::IV_SIZE +	EncPayload::MIN_ENCAP_PAYLOAD_SIZE +	EncPayload::CMAC_SIZE;
		static const size_t MAX_ENCAP_PAYLOAD_SIZE = EncPayload::MAX_SIZE - EncPayload::IV_SIZE - EncPayload::CMAC_SIZE - 1;
 		
 
		uint8_t m_iv[EncPayload::IV_SIZE];
		uint8_t m_payload[EncPayload::MAX_ENCAP_PAYLOAD_SIZE];
		size_t m_payloadLen;
		uint8_t m_nonceId;
		uint8_t m_cmac[EncPayload::CMAC_SIZE];		

		EncPayload ();
		virtual ~EncPayload ();
		size_t Pack (uint8_t *buf, size_t maxLen);
		size_t Unpack (uint8_t *buf, size_t maxLen);
		size_t GetPackedLen ();
};

class Security : APP{
public:
	static const uint8_t CMD_CLASS = 0x98;
	static const uint8_t CMD_NONCEGET = 0x40;
	static const uint8_t CMD_NONCEREP = 0x80;
	static const uint8_t CMD_ENCENCRT = 0x81;

	static const size_t NONCE_SIZE = 8;
	static const size_t BLOCK_SIZE = 16;
	static const size_t NONCE_REQUEST_FRAME_SIZE = sizeof(MAC::mhr_single_t) + sizeof(APP::ahr_t) + MAC::CHECKSUM_SIZE;
	static const size_t NONCE_REPORT_FRAME_SIZE = sizeof(MAC::mhr_single_t) + sizeof(APP::ahr_t) + NONCE_SIZE + MAC::CHECKSUM_SIZE;

	struct mac_message_t{
		static const size_t MAX_PAYLOAD_SIZE = EncPayload::MAX_ENCAP_PAYLOAD_SIZE;
		static const size_t MESSAGE_HDR_SIZE = 4;
		static const size_t MAX_MESSAGE_SIZE = MAX_PAYLOAD_SIZE + MESSAGE_HDR_SIZE;

		uint8_t sh;
		uint8_t src;
		uint8_t dst;
		uint8_t len;
		uint8_t payload[MAX_PAYLOAD_SIZE];

		size_t getLength(){ return len + MESSAGE_HDR_SIZE; } // Hardcoded size :(
	}__attribute__((packed));

	struct nonce_rep_t {
		uint8_t nonce[Security::NONCE_SIZE];

		uint8_t GetNonceID () { return nonce[0]; }
	}__attribute__((packed));

	static size_t GetNonceReport (nonce_rep_t &nr, uint8_t *app, size_t app_len);
	static void GetMACMessage (mac_message_t &mm, uint8_t cmd, uint8_t src, uint8_t dst, EncPayload *ep);

	// These were originally members of EncapPayload, but there was something ugly about that. I guess it was having heavy encryption stuff in a class about a payload. I also didn't want to have to allocate encryption buffers for each encap object.
	static bool Decrypt (EncPayload &ep, uint8_t *enc_key, uint8_t *nonce);
	static bool Encrypt (EncPayload &ep, uint8_t *enc_key, uint8_t *nonce);
	static bool GenerateCMAC (EncPayload &ep, uint8_t *mac_key, uint8_t *nonce, uint8_t cmd, uint8_t src, uint8_t dst);
	static bool CheckCMAC (EncPayload *ep, uint8_t *mac_key, uint8_t *nonce, uint8_t cmd, uint8_t src, uint8_t dst);

private:
		// Other CMAC public members call this as a primative to create the CMACs
  static bool GetCMAC (uint8_t *cmac, mac_message_t *macMsg, uint8_t *macKey, uint8_t *iv);
	//static bool GetCMAC2 (uint8_t *cmac, mac_message_t *macMsg, uint8_t *macKey, uint8_t *iv);
	static void PrintHex (std::string header, uint8_t *buf, size_t len);
};

}
#endif
