#include "SecurityHelpers.h"
#include <cstring>
#include <stdint.h>
#include <cryptopp/modes.h>
#include <cryptopp/aes.h>
#include <cryptopp/filters.h>
#include <iostream>

namespace Zwave{
KeyRing::KeyRing ()
{
	std::memset(m_net, 0, KeyRing::KEY_LEN);
	std::memset(m_mac, 0, KeyRing::KEY_LEN);
	std::memset(m_enc, 0, KeyRing::KEY_LEN);
}

KeyRing::KeyRing (uint8_t *network_key)
{
	std::memcpy(m_net, network_key, KeyRing::KEY_LEN);
	GenerateFromNetworkKey ();
}

KeyRing::~KeyRing(){}

void 
KeyRing::GenerateFromNetworkKey ()
{
	std::string plainTxt, encryptedTxt;
	CryptoPP::ECB_Mode< CryptoPP::AES >::Encryption e;

	try
		{
			e.SetKey( m_net, KEY_LEN);

			// For encryption key					
			std::memset (m_block, ENC_PASSWORD, KEY_LEN );
			plainTxt = std::string ((const char*)m_block, BLOCK_SIZE);
 			encryptedTxt.clear ();
			CryptoPP::StringSource ss1( plainTxt, true, new CryptoPP::StreamTransformationFilter( e, new CryptoPP::StringSink(encryptedTxt ))); // StringSource
			std::memcpy (m_enc, (uint8_t*)encryptedTxt.c_str (), KEY_LEN);

			// For auth key 
		//There is probably a more efficient way to do this in crypto++ because it seems very clumsy to have to generate a sink source pair for every decryption operation. TODO?

			std::memset (m_block, MAC_PASSWORD, KEY_LEN );
			plainTxt = std::string ((const char*)m_block, BLOCK_SIZE);
 			encryptedTxt.clear ();
			CryptoPP::StringSource ss2( plainTxt, true, new CryptoPP::StreamTransformationFilter( e, new CryptoPP::StringSink(encryptedTxt ))); // StringSource
			std::memcpy (m_mac, (uint8_t*)encryptedTxt.c_str (), KEY_LEN);
	}
	catch( CryptoPP::Exception& e )
		{
				std::cerr << e.what() << std::endl;
				std::exit(1);
		}
}

NonceCache::NonceCache ()
{
	m_ctr = 0;
}

NonceCache::~NonceCache () {}

uint8_t* 
NonceCache::FindNonce (uint8_t nonceId)
{
	size_t index = GetNonceIndex (nonceId);
	if (index < CACHE_SIZE)
		return &m_cache[index][0];
	else
		return NULL;
}

size_t
NonceCache::StoreNonce (uint8_t *nonce)
{
	size_t index = m_ctr % CACHE_SIZE;
	std::memcpy(m_cache[index], nonce, Security::NONCE_SIZE);
	m_ctr++;

	return index;
}

bool
NonceCache::isInCache (uint8_t nonceId)
{
	if (GetNonceIndex (nonceId) < CACHE_SIZE)
		return true;
	else
		return false;
}

size_t
NonceCache::GetNonceIndex (uint8_t nonceId)
{
	size_t limit;

	if (m_ctr > CACHE_SIZE)
		limit = CACHE_SIZE;
	else
		limit = m_ctr;

	for(size_t i=0; i<limit;i++)
		{
			if (m_cache[i][0] == nonceId)
				return i;
		}

	return CACHE_SIZE; // Out of bounds.
}

}
