#ifndef _SECURITY_HELPERS_H_
#define _SECURITY_HELPERS_H_

#include <cryptopp/aes.h>
#include <stdint.h>
#include "Security.h"

namespace Zwave {
	class KeyRing {
		public:
			static const size_t BLOCK_SIZE = CryptoPP::AES::BLOCKSIZE;
			static const size_t KEY_LEN = CryptoPP::AES::DEFAULT_KEYLENGTH;
			uint8_t m_net[KEY_LEN];
			uint8_t m_mac[KEY_LEN];
			uint8_t m_enc[KEY_LEN];
	
			KeyRing ();
			KeyRing (uint8_t *network_key);
			virtual ~KeyRing();

			void GenerateFromNetworkKey ();

		private:
			uint8_t m_block[BLOCK_SIZE];
			static const uint8_t ENC_PASSWORD = 0xAA;
			static const uint8_t MAC_PASSWORD = 0x55;
	};

	class NonceCache {
		public:
			static const size_t CACHE_SIZE = 10;

			NonceCache ();
			virtual ~NonceCache ();
			uint8_t* FindNonce (uint8_t nonceId);
			size_t StoreNonce (uint8_t *nonce);
			bool isInCache (uint8_t nonceId);
 
		private:
			size_t GetNonceIndex (uint8_t nonceId);
			uint8_t	m_cache[CACHE_SIZE][Security::NONCE_SIZE];
			size_t m_ctr;
	};
}
#endif
