#include "Timer.h"
#include <sys/time.h>
#include <cstring>
#include <string>
#include <stdint.h>
#include <iostream>

namespace Zwave{

Timer::Timer ()
{
	std::memset (&m_deadline, 0, sizeof(struct timeval));
}

Timer::Timer (struct timeval *delay)
{
	SetAsDelay(delay);
}

Timer::Timer (long sec, long usec)
{
	struct timeval delay;
	delay.tv_sec = sec;
	delay.tv_usec = usec;

	SetAsDelay (&delay);
}

Timer::~Timer () {}

/*
void
Timer::SetAsDeadline (struct timeval *deadline)
{
	std::memcpy (&m_deadline, deadline, sizeof(struct timeval));
	return;
}
*/

void
Timer::SetAsDelay (struct timeval *delay)
{
	int carry;
	
	//PrintTimeVal ("SetDelay::Delay", delay);
	gettimeofday (&m_deadline, NULL);
	//PrintTimeVal ("SetDelay::Now", &m_deadline);

	m_deadline.tv_usec += delay->tv_usec;
	
	carry = m_deadline.tv_usec / Timer::USEC_IN_SEC;
	m_deadline.tv_usec = m_deadline.tv_usec % Timer::USEC_IN_SEC;
	m_deadline.tv_sec += delay->tv_sec;
	m_deadline.tv_sec += carry;

	//PrintTimeVal ("SetDelay::Deadline", &m_deadline);
	return;
}

void
Timer::PrintTimeVal (std::string header, struct timeval *tv)
{
	std::cout << header << ":" << tv->tv_sec << "." << tv->tv_usec << std::endl;
}

bool
Timer::HasTimeExpired ()
{
	struct timeval now;
	bool expired = false;

	gettimeofday(&now, NULL);

	//PrintTimeVal("HasTimeExpired::Deadline", &m_deadline);
	//PrintTimeVal("HasTimeExpired::Now", &now);

	if (now.tv_sec > m_deadline.tv_sec)
		expired = true;
	else if ((now.tv_sec == m_deadline.tv_sec) && (now.tv_usec > m_deadline.tv_usec))
		expired = true;

	//std::cout << "Expired=" << expired << std::endl;	
	return expired;
}
	
void
Timer::GetRemainingTime (struct timeval &timeLeft)
{
	struct timeval now;

	gettimeofday (&now, NULL);
	std::memset (&timeLeft, 0, sizeof(struct timeval));

	//PrintTimeVal ("GetTimeLeft::Deadline", &m_deadline);
	//PrintTimeVal ("GetTimeLeft::Now", &now);

	if (m_deadline.tv_sec > now.tv_sec)
		{
			timeLeft.tv_sec = m_deadline.tv_sec - now.tv_sec;
			if (m_deadline.tv_usec >= now.tv_usec)
				{
					timeLeft.tv_usec = m_deadline.tv_usec - now.tv_usec;
				}
			else
				{
					timeLeft.tv_sec--;
					timeLeft.tv_usec = (m_deadline.tv_usec + Timer::USEC_IN_SEC) - now.tv_usec;
				}
		}
	else if (m_deadline.tv_sec == now.tv_sec)
		{
			if (m_deadline.tv_usec >= now.tv_usec)
				{
					timeLeft.tv_usec = m_deadline.tv_usec - now.tv_usec;
				} // else there is no time left (less than one second has passed since deadline)
		} // else there is no time left (more than one second has passed since the deadline
	
	//PrintTimeVal ("GetTimeLeft::Timeleft", &timeLeft);
	return;
}

void 
Timer::GetDeadline (struct timeval &deadline)
{
	std::memcpy (&deadline, &m_deadline, sizeof(struct timeval));
	return;
}

}
