/*
 * ScapyIface.cpp
 *
 *  Created on: Feb 1, 2016
 *      Author: badenhopcw
 */

#include "UdpSocketIface.h"
#include <string>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <cstring>
#include <stdio.h>
#include <stdlib.h>

namespace Zwave {

UdpSocketServerIface::UdpSocketServerIface(std::string ipAddress, uint16_t port, int flags) {
	// TODO Auto-generated constructor stub

	m_flags = flags;
	m_sockLen = sizeof(m_sockAddr);

	if ((m_socket = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1){
		perror("socket create failed");
		exit(1);
	}
	
	std::memset((char *) &m_sockAddr, 0, m_sockLen);	
	m_sockAddr.sin_family = AF_INET;
	m_sockAddr.sin_port = ::htons(port);

  if (::inet_aton(ipAddress.c_str () , &m_sockAddr.sin_addr)==0) {
     perror("inet_aton() failed");
		 exit(1);
	}
	
	if (::bind (m_socket, (sockaddr *)&m_sockAddr, m_sockLen) == -1 ) {
		perror ("bind error");
		exit(1);
	}
}

int
UdpSocketServerIface::GetFlags ()
{
	return m_flags;
}

void
UdpSocketServerIface::SetFlags (int f)
{
	m_flags = f;
}

int
UdpSocketServerIface::GetSocket ()
{
	return m_socket;
}

UdpSocketServerIface::UdpSocketServerIface() {
	m_socket = -1;
}

UdpSocketServerIface::~UdpSocketServerIface() {
	//if (m_socket >= 0)
	//	::close(m_socket);
}

ssize_t
UdpSocketServerIface::RecvFrom (uint8_t *buf, size_t len)
{
	return ::recvfrom (m_socket, (void*)buf, len, m_flags, NULL, NULL);
}

UdpSocketClientIface::UdpSocketClientIface(std::string ipAddress, uint16_t port, int flags) {
	// TODO Auto-generated constructor stub

	m_flags = flags;

	m_sockLen = sizeof(m_sockAddr);

	if ((m_socket = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP))==-1){
		perror("socket create failed");
		exit(1);
	}

	memset((char *) &m_sockAddr, 0, m_sockLen);
	m_sockAddr.sin_family = AF_INET;
	m_sockAddr.sin_port = ::htons(port);

  if (::inet_aton(ipAddress.c_str () , &m_sockAddr.sin_addr)==0) {
     perror("inet_aton() failed");
		 exit(1);
	}
	
}

UdpSocketClientIface::UdpSocketClientIface() {
	m_socket = -1;
}

UdpSocketClientIface::~UdpSocketClientIface() {
	//if (m_socket >= 0)
	//	::close(m_socket);
}

ssize_t
UdpSocketClientIface::SendTo (const uint8_t *buf, size_t len)
{
	return ::sendto (m_socket, (void*)buf, len, m_flags, (sockaddr*)&m_sockAddr, sizeof(m_sockAddr));
}

int
UdpSocketClientIface::GetFlags ()
{
	return m_flags;
}

void
UdpSocketClientIface::SetFlags (int f)
{
	m_flags = f;
}

int
UdpSocketClientIface::GetSocket ()
{
	return m_socket;
}
} /* namespace Zwave */
