#include "gcrypt.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define BLOCK_LEN 16

void PrintHex (char *header, uint8_t *buf, size_t len){
	size_t i;
	printf("%s: ", header);
	for(i=0;i<len;i++){
		printf("%x ", buf[i]);
	}
	printf("\n");
	return;
}

int main(int argc, char *argv[]){
	
//	const uint8_t network_key[BLOCK_LEN] = {0x07,0x61,0xeb,0x2c,0xdb,0xe1,0xb3,0xc7,0x72,0x0e,0xfd,0x3d,0x13,0xd7,0x86,0x1b};
//	const uint8_t auth_key[BLOCK_LEN] = {0x68,0xa4,0xa7,0x38,0xff,0xaa,0x27,0xb8,0x7f,0x81,0x20,0xdf,0xc3,0x1f,0x39,0x84};
//	const uint8_t encrypt_key[BLOCK_LEN] = {0xfc,0xcb,0x6b,0xee,0x60,0x3a,0xca,0xfd,0xf5,0xa8,0xae,0x13,0x17,0xb0,0x35,0x8e};

	uint8_t network_key[BLOCK_LEN];
	uint8_t auth_key[BLOCK_LEN];
	uint8_t encrypt_key[BLOCK_LEN];

	uint8_t inBlock[BLOCK_LEN];
	uint8_t outBlock[BLOCK_LEN];
	int i;
	gcry_error_t err;
	gcry_cipher_hd_t hd;
	
	int fd;

	if (argc != 2){
		printf("Usage is %s key_file\n", argv[0]);
		exit(1);
	}

	fd = open (argv[1], O_RDONLY, 0);
	read (fd, network_key, BLOCK_LEN);
	read (fd, auth_key, BLOCK_LEN);
	read (fd, encrypt_key, BLOCK_LEN);
  
/* Version check should be the very first call because it
     makes sure that important subsystems are intialized. */
  if (!gcry_check_version (GCRYPT_VERSION))
    {
      fputs ("libgcrypt version mismatch\n", stderr);
      exit (2);
    }

  /* Disable secure memory.  */
  gcry_control (GCRYCTL_DISABLE_SECMEM, 0);

  /* ... If required, other initialization goes here.  */

  /* Tell Libgcrypt that initialization has completed. */
  gcry_control (GCRYCTL_INITIALIZATION_FINISHED, 0);
	err = gcry_cipher_open(&hd, GCRY_CIPHER_AES128, GCRY_CIPHER_MODE_ECB, 0);
	if (err)
    {
        printf("gcry_cipher_open failed:  %s/%s\n",
               gcry_strsource(err),
               gcry_strerror(err));
        
    }
	else printf("gcry_cipher_open worked\n");


	err = gcry_cipher_setkey(hd, network_key, 16);
	if (err)
    {
        printf("gcry_cipher_setkey failed:  %s/%s\n",
               gcry_strsource(err),
               gcry_strerror(err));
        
    }
	else printf("gcry_cipher_setkey worked\n");

	// Prepare the blocks for decrypting
	memcpy(inBlock, auth_key, BLOCK_LEN);
	memset(outBlock, 0, BLOCK_LEN);

	// do it
	err = gcry_cipher_decrypt(hd, outBlock, BLOCK_LEN, inBlock, BLOCK_LEN);
	if (err)
    {
        printf("gcry_cipher_decrypt failed:  %s/%s\n",
               gcry_strsource(err),
               gcry_strerror(err));
        
    }
	else printf("gcry_cipher_decrypt worked\n");

	PrintHex ("A_K: ", inBlock, BLOCK_LEN);
	PrintHex ("A_Seed: ", outBlock, BLOCK_LEN);


// For encrypt key...do we ever need to reset ECB? I doubt it.
	memcpy(inBlock, encrypt_key, BLOCK_LEN);
	memset(outBlock, 0, BLOCK_LEN);
 
// do it
	err = gcry_cipher_decrypt(hd, outBlock, BLOCK_LEN, inBlock, BLOCK_LEN);
	if (err)
    {
        printf("gcry_cipher_decrypt failed:  %s/%s\n",
               gcry_strsource(err),
               gcry_strerror(err));
        
    }
	else printf("gcry_cipher_decrypt worked\n");

	PrintHex ("E_K: ", inBlock, BLOCK_LEN);
	PrintHex ("E_Seed: ", outBlock, BLOCK_LEN);

/*
	printf("Decrypting 0xAA with ENC key\n");
	// Lets try a fun experiment
	err = gcry_cipher_setkey(hd, encrypt_key, 16);
	memset(inBlock, 0xAA, BLOCK_LEN);
	memset(outBlock, 0, BLOCK_LEN);
	err = gcry_cipher_decrypt(hd, outBlock, BLOCK_LEN, inBlock, BLOCK_LEN);
	PrintHex ("Result: ", outBlock, BLOCK_LEN);

	printf("Encrypting 0xAA with ENC key\n");
	memset(outBlock, 0, BLOCK_LEN);
	err = gcry_cipher_encrypt(hd, outBlock, BLOCK_LEN, inBlock, BLOCK_LEN);
	PrintHex ("Re: ", outBlock, BLOCK_LEN);

	printf("Decrypting ENC Key with 0xAA as key\n");
	memset(outBlock, 0, BLOCK_LEN);
	memcpy(inBlock, encrypt_key, 16);
	memset(encrypt_key, 0xAA, 16); // This will destroy the key we already found, so anything after this wont work until key is restored.
	err = gcry_cipher_setkey(hd, encrypt_key, 16);
	err = gcry_cipher_decrypt(hd, outBlock, BLOCK_LEN, inBlock, BLOCK_LEN);
PrintHex ("Re: ", outBlock, BLOCK_LEN);

	memset(outBlock, 0, BLOCK_LEN);
	err = gcry_cipher_encrypt(hd, outBlock, BLOCK_LEN, inBlock, BLOCK_LEN);
	PrintHex ("Re: ", outBlock, BLOCK_LEN);
*/

	gcry_cipher_close(hd);
	
	return 0;
}
